/**
 * \file gre/uicad.h
 * \brief GRE User Interface definitions for CAD layer
 *
 * \if NODOC
 * $Id: uicad.h_v 1.6 2013/02/15 20:48:51 mju Exp $
 *
 * $Log: uicad.h_v $
 * Revision 1.6  2013/02/15 20:48:51  mju
 * Fix dox hdr.
 *
 * Revision 1.5  2011/10/20 20:26:08  mju
 * Remove editInTNTserver flag.
 *
 * Revision 1.4  2006/08/31 17:16:49  mju
 * Add onClose.
 *
 * Revision 1.3  2006/06/28 17:20:46  mju
 * Fix header doxy comment.
 *
 * Revision 1.2  2005/10/17 15:28:36  mju
 * Use gre namespace.
 *
 * Revision 1.1  2004/06/16 21:04:46  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_UICAD_H
#define	INC_GRE_UICAD_H

#ifndef	INC_GRE_LRCAD_H
#include <gre/lrcad.h>
#endif

#ifdef USE_MGUI

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace GRE {
//===================================================================================================================
//! Dialog for modifying vector layer display settings.
class LAYER_CAD::DISPPARM::DLG : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG (
			LAYER_CAD* CadLayer,
			bool Include3D
			);

		//! Destructor.
		virtual ~DLG (
			);

		//! Get 'mask' for which LAYERDRAWFLAG values are settable on dialog.
		LAYERDRAWFLAGS GetLayerDrawFlagMask (
			) const { return (LAYERDRAWFLAG_None); }

		//! Get current LAYERDRAWFLAGS settings.
		LAYERDRAWFLAGS GetLayerDrawFlags (
			) const;

		//! Get current DISPPARM values.
		const LAYER_CAD::DISPPARM& GetValues (
			) const;

		//! Set DISPPARM and LAYERDRAWFLAGS values.
		ERRVALUE SetValues (
			const LAYER_CAD::DISPPARM& values,
			LAYERDRAWFLAGS LayerDrawFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		LAYER_CAD *m_layer;

		void ApplyChanges (bool ForceRedraw);

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual void v_OnOpen ();

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// USE_MGUI

#endif	// INC_GRE_UICAD_H

