/**
 * \file gre/uiformview.h
 * \brief GRE FORM_VIEW class definition
 *
 * \if NODOC
 * $Id: uiformview.h_v 1.50 2013/01/30 16:21:13 mju Exp $
 *
 * $Log: uiformview.h_v $
 * Revision 1.50  2013/01/30 16:21:13  mju
 * Remove zoomSetHeight.
 *
 * Revision 1.49  2012/12/05 16:49:45  mju
 * Add optionsmenu enum and use in menuAddSubMenuOptions.
 *
 * Revision 1.48  2012/05/30 15:31:34  mju
 * Allow to specify viewable in create instead of constructor.
 *
 * Revision 1.47  2012/05/17 16:12:17  mju
 * Add submenus enum, menuBarAddSubMenus and parm to menuBarCreate.
 *
 * Revision 1.46  2012/04/06 14:30:40  mju
 * Change method names for stereo view indicator.
 *
 * Revision 1.45  2012/04/04 20:48:53  mju
 * Methods for stereo mode indicator visibility.
 *
 * Revision 1.44  2012/03/26 17:58:35  mju
 * In statusDisplayCoord use flags and optional z unit.
 *
 * Revision 1.43  2011/12/20 18:39:30  mju
 * Add createflag_NoReadSettingsIni.
 *
 * Revision 1.42  2011/11/29 21:14:48  mju
 * Remove viewbtn_zoomToLocation.
 *
 * Revision 1.41  2011/11/29 18:22:13  mju
 * Add viewbtn_printSnapshot.
 *
 * Revision 1.40  2011/11/28 17:13:00  mju
 * Fix dox dependency.
 *
 * Revision 1.39  2011/07/11 19:56:58  bcolombe
 * Add zoom sidebar panel
 *
 * Revision 1.38  2011/04/14 20:38:18  bcolombe
 * Add SetLayerListPos
 *
 * Revision 1.37  2011/03/09 16:49:27  mju
 * Add createflag_NoScalebarGadget.
 *
 * Revision 1.36  2011/02/21 21:55:08  bcolombe
 * Add places tool
 *
 * Revision 1.35  2010/11/17 18:21:37  mju
 * Add ability to include layer manager button on toolbar.
 *
 * Revision 1.34  2010/10/28 16:02:50  mju
 * Add menuGetOptions so can add to options menu.
 *
 * Revision 1.33  2010/03/08 23:25:50  mju
 * Add checkSidebar to ...IsVisible.
 *
 * Revision 1.32  2010/03/04 22:44:28  mju
 * Move guts to "PRIV" class so can change without affecting dependent code.
 * Add "sidebar" and ability to control legend visibility.
 *
 * Revision 1.31  2010/02/26 01:35:21  bcolombe
 * Added delegate for magnifier resizing
 *
 * Revision 1.30  2010/02/25 17:03:39  bcolombe
 * Added delegate for magnifier zoom
 *
 * Revision 1.29  2010/02/25 13:49:07  mju
 * Forward declare ctrl_magnifier.
 *
 * Revision 1.28  2010/02/25 00:14:59  bcolombe
 * Use CTRL_MAGNIFIER
 *
 * Revision 1.27  2010/02/24 22:22:22  mju
 * Use layout_pane_sash.
 *
 * Revision 1.26  2010/02/22 17:25:26  bcolombe
 * Added magnifier
 *
 * Revision 1.25  2010/01/26 18:47:03  mju
 * Add setDrawWireframe3D.
 *
 * Revision 1.24  2009/11/25 16:45:09  mju
 * Fix doxygen warning.
 *
 * Revision 1.23  2009/02/06 15:01:05  mju
 * Add statusPosGetCoordSource.
 *
 * Revision 1.22  2008/12/15 17:37:12  mju
 * Add toolbarHndAddLayer and methods for "add layer" toolbar button.
 *
 * Revision 1.21  2008/10/21 16:56:05  bcolombe
 * Added createflag AllowGPS to construct GPSINFO class without making menu for it
 * Otherwise gps wouldn't work as it would only be constructed from MenuBarCreate called in SHELL_VIEW
 *
 * Revision 1.20  2008/09/12 21:43:14  mju
 * Add "addSeparator" parm to menuBarCreate.
 *
 * Revision 1.19  2008/05/19 17:02:58  mju
 * Add statubar item index for background job status.
 *
 * Revision 1.18  2008/04/28 20:33:44  mju
 * Add dlgCustomize methods.
 *
 * Revision 1.17  2008/04/25 14:19:19  mju
 * Add gpsinfo member, remove unused gps methods.
 *
 * Revision 1.16  2008/04/18 17:30:55  mju
 * Add createflag_isLocator.
 *
 * Revision 1.15  2008/04/18 16:27:24  mju
 * Add getPaneToolBar.
 *
 * Revision 1.14  2008/03/06 16:17:49  mju
 * Add createflag_NoDftButtons.
 *
 * Revision 1.13  2008/02/22 22:37:53  mju
 * Support saving layout settings.
 *
 * Revision 1.12  2008/02/12 22:28:06  mju
 * Add sml_createForm and sml_GetCreateFlags.
 *
 * Revision 1.11  2008/02/07 16:02:00  mju
 * Add createflag_noToolScripts.
 *
 * Revision 1.10  2008/02/06 22:33:46  mju
 * Begin add toolscript support.
 *
 * Revision 1.9  2008/02/06 21:08:34  mju
 * Add macroscript support.
 *
 * Revision 1.8  2008/02/01 18:05:07  mju
 * More macroscript stuff.
 *
 * Revision 1.7  2008/01/31 15:36:08  mju
 * Add parm to statusUpdatePos for controlling report precision.
 *
 * Revision 1.6  2008/01/25 22:10:39  mju
 * Add locator view.
 *
 * Revision 1.5  2008/01/18 18:47:54  mju
 * Add 3d form and toolbar.
 *
 * Revision 1.4  2007/12/28 18:54:48  mju
 * Combine position/scale and status bar.
 *
 * Revision 1.3  2007/12/06 23:01:22  mju
 * Change how determines if legend is visible.
 *
 * Revision 1.2  2007/12/03 18:57:05  mju
 * Add handlers for pointer tracking.
 *
 * Revision 1.1  2007/12/03 16:52:13  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_UIFORMVIEW_H
#define	INC_GRE_UIFORMVIEW_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
	#include <mi32/latlonfm.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace MGUI {
	class CTRL_STATUSBAR;
	class CTRL_MAGNIFIER;
	class DLG_CUSTOMIZE;
	class MENU;
	class SHELL;
	}
namespace SPATREF {
	class COORDREFSYS;
	}
namespace GRE {
	namespace UI {
		class VIEW_CANVAS;
		}
	}
namespace SML {
	union ARG;
	class CONTEXT;
	class FLAG;
	}
namespace UOM {
	class UNIT;
	}
#endif


namespace GRE {
namespace UI {

//===================================================================================================================

class FORM_VIEW : public MGUI::FORM_COMPOSITE {
	public:

		//! Flags for form creation.
		enum CREATEFLAGS {
			CREATEFLAG_None =							0x00000000,
			// Flags must match GRE::VIEW::CREATEFLAGS.
			CREATEFLAG_3D =							0x00000001,	//!< View is 3D
			CREATEFLAG_NoCallCreateCB =			0x00000002,	//!< Don't call callback list when object created
			CREATEFLAG_NoRenderSeparate =			0x00000004,	//!< Don't render to separate memory buffers
			CREATEFLAG_DisableDraw =				0x00000010,	//!< Start with drawing initially disabled
			CREATEFLAG_IsLocator =					0x00000020,	//!< Create as "locator" view
			CREATEFLAG_AllowGPS =					0x00000040, //!< Allow gps without making menu.  Not needed for a SHELL_VIEW

			CREATEFLAG_NoDftViewButtons =			0x00000100,	//!< Don't put default "view" buttons on toolar
			CREATEFLAG_NoDftAddButtons =			0x00000200,	//!< Don't put default "add" button(s) on toolbar
			CREATEFLAG_NoDftMarkButtons =			0x00000400,	//!< Don't put default "mark" buttons on toolar
			CREATEFLAG_NoDftTools =					0x00000800,	//!< Don't add default tools
			CREATEFLAG_NoDft3dButtons =			0x00001000,	//!< Don't put default "3D" buttons on toolar (for 3D group view)
			CREATEFLAG_NoToolScripts =				0x00002000,	//!< Don't add ToolScripts
			CREATEFLAG_NoDftButtons =				0x00004000,	//!< Don't add any default buttons to toolbar
			CREATEFLAG_NoSidebar =					0x00008000,	//!< Don't create sidebar pane
			CREATEFLAG_NoLegend =					0x00010000,	//!< Don't create legend pane
			CREATEFLAG_NoLocator =					0x00020000,	//!< Don't create locator pane
			CREATEFLAG_NoStatusBar =				0x00040000,	//!< Don't create statusbar pane
			CREATEFLAG_NoToolBar =					0x00080000,	//!< Don't create toolbar
			CREATEFLAG_No3dToolBar =				0x00100000,	//!< Don't create 3D toolbar (for 3D group view)
			CREATEFLAG_NoMagnifier =				0x00200000,	//!< Don't create magnifier pane
			CREATEFLAG_AddLayerMgrButton =		0x00400000,	//!< Include "layer manager" button on toolbar (group view only), cannot use with CREATEFLAG_NoDftAddButtons
			CREATEFLAG_NoPlaces =					0x00800000,	//!< Don't create places pane
			CREATEFLAG_NoScalebarGadget =			0x01000000,	//!< Don't include scalebar gadget
			CREATEFLAG_NoZoomPane =					0x02000000,	//!< Don't create zoom pane
			CREATEFLAG_NoReadSettingsIni =		0x04000000,	//!< Don't read settings from Ini file when not provided

			CREATEFLAG_AllowGeoLock =				0x10000000	//!< Allow GeoLocking
			};

		//! Buttons for "View" actions.
		enum VIEWBTNS {
			VIEWBTN_Refresh =				0x00000001,
			VIEWBTN_Stop =					0x00000002,
			VIEWBTN_Full =					0x00000004,
			VIEWBTN_Previous =			0x00000008,
			VIEWBTN_ZoomIn =				0x00000020,
			VIEWBTN_ZoomOut =				0x00000040,
			VIEWBTN_ZoomToGroup =		0x00000080,
			VIEWBTN_ZoomToLayer =		0x00000100,
			VIEWBTN_ZoomToMaxDetail =	0x00000200,
			VIEWBTN_Zoom1X =				0x00000800,
			VIEWBTN_Zoom2X =				0x00001000,
			VIEWBTN_Zoom3X =				0x00002000,
			VIEWBTN_Locator =				0x00004000,
			VIEWBTN_Snapshot =			0x00008000,
			VIEWBTN_GeoLock =				0x00010000,
			VIEWBTN_SyncGeoViewer =		0x00020000,		//!< Synchronize external GeoViewer to current view
			VIEWBTN_Stereo =				0x00040000,
			VIEWBTN_PrintSnapshot =		0x00080000,
			VIEWBTN_Default =				VIEWBTN_Refresh | VIEWBTN_Stop | VIEWBTN_Full | VIEWBTN_Previous | VIEWBTN_ZoomIn | VIEWBTN_ZoomOut | VIEWBTN_ZoomToLayer | VIEWBTN_ZoomToGroup | VIEWBTN_ZoomToMaxDetail | VIEWBTN_Snapshot | VIEWBTN_GeoLock | VIEWBTN_SyncGeoViewer | VIEWBTN_Stereo
			};

		enum COORDSOURCE {
			COORDSOURCE_None,
			COORDSOURCE_Pointer,
			COORDSOURCE_ViewCenter,
			COORDSOURCE_Entered,
			COORDSOURCE_GPS,
			COORDSOURCE_COUNT
			};

		enum STATUSDISPLAY {
			STATUSDISPLAY_Default =						0x00,
			STATUSDISPLAY_UseLatLonIfNoSetCRS =		0x01,	//!< Use latitude-longitude if no CRS set for status
			STATUSDISPLAY_HasZ =							0x02,	//!< Z value is valid
			};

		//! Submenus that can be added to menubar.
		enum SUBMENUS {
			SUBMENU_View =				0x01,
			SUBMENU_Tools =			0x02,
			SUBMENU_GPS =				0x04,
			SUBMENU_Options =			0x08,
			SUBMENU_Macros =			0x10,
			SUBMENU_HotKeys =			0x20,
			SUBMENUS_All =				0xFF
			};

		enum OPTIONSMENU {
			OPTIONSMENU_Default =			0,
			OPTIONSMENU_NoColors =			0x0001,
			OPTIONSMENU_NoStereo =			0x0002,
			OPTIONSMENU_NoDataTips =		0x0004,
			OPTIONSMENU_NoProjAccuracy =	0x0008,
			OPTIONSMENU_NoRedrawToggle =	0x0010,
			OPTIONSMENU_NoDiffZoom =		0x0020,
			OPTIONSMENU_NoCustomize =		0x0040,
			};

		class SETTINGS;

		typedef fastdelegate::FastDelegate<void(UI::VIEW_CANVAS*)> DELEGATE_ADDTOOLS;

		//! Default constructor.
		//! If this is used, must specify viewable in Create().
		FORM_VIEW ();

		//! Constructor with existing VIEWABLE.
		explicit FORM_VIEW (
			GRE::VIEWABLE *viewable						//!< Existing viewable
			);

		//! Destructor.
		virtual ~FORM_VIEW ();

		//! Clear message line.
		void ClearMessage ();

		//! Clear previous view list.
		void ClearPreviousViewList ();

		//! Create form with viewable having been specified in constructor.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& PaneParent,
			INT32 CanvasWidth,							//!< Initial canvas width in pixels
			INT32 CanvasHeight,							//!< Initial canvas height in pixels
			const char *IniFieldBase = 0,				//!< Field from .ini for auto-savable settings, 0 for none
			CREATEFLAGS CreateFlags = CREATEFLAG_None,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			const SETTINGS *pSettings = 0
			);

		//! Create form with specified viewable.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& PaneParent,
			GRE::VIEWABLE *viewable,					//!< Existing viewable
			INT32 CanvasWidth,							//!< Initial canvas width in pixels
			INT32 CanvasHeight,							//!< Initial canvas height in pixels
			const char *IniFieldBase = 0,				//!< Field from .ini for auto-savable settings, 0 for none
			CREATEFLAGS CreateFlags = CREATEFLAG_None,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			const SETTINGS *pSettings = 0
			);

		//! Create 3D toolbar.
		//! Used if want to create in nonstandard position.
		void Create3dToolBar (
			MGUI::LAYOUT_PANE& PaneParent				//!< Parent pane
			);

		//! Create pane for toolbar.
		//! Used if want to create in nonstandard position.
		void CreatePaneToolBar (
			MGUI::LAYOUT_PANE& PaneParent				//!< Parent pane
			);

		//! Create statusbar.
		//! Used if want to create in nonstandard position.
		void CreateStatusBar (
			MGUI::LAYOUT_PANE& PaneParent,
			const SETTINGS *pSettings = 0
			);

		//! Disable drawing for this view.
		void DisableDraw (
			);

		//! Add predefined menu items to customization dialog.
		void DlgCustomizeAddMenuItems (
			MGUI::DLG_CUSTOMIZE& dlg
			);

		//! Add predefined toolbar items to customization dialog.
		void DlgCustomizeAddToolBarItems (
			MGUI::DLG_CUSTOMIZE& dlg
			);

		//! Edit background, highlight, active colors for view.
		//! @return TRUE if colors changed, FALSE if not, < 0 if error.
		int DlgEditColors (
			);

		//! Edit interface customization settings for views of this type.
		void DlgEditCustomization (
			);

		//! Draw view.
		ERRVALUE Draw (
			);

		//! Draw view if auto-update is on.
		ERRVALUE DrawIfAutoUpdate (
			);

		//! Draw if changed since last redraw.
		//! @return TRUE if did redraw, FALSE if not, < 0 if error.
		int DrawIfNeeded (
			);

		//! Enable drawing for this view.
		void EnableDraw (
			);

		//! Get "magnifier" managed by form.
		MGUI::CTRL_MAGNIFIER* GetCanvasMagnifier (
			) const;

		//! Get LAYOUT_PANE for toolbar.
		MGUI::LAYOUT_PANE& GetPaneToolBar (
			);

		//! Determine if should redraw views on any change
		bool GetRedrawOnAnyChange (
			) const;

		//! Get settings to save.
		//! This will update settings from current form and view canvas, so should not be used
		//! to access individual values.
		const SETTINGS& GetSettingsToSave (
			);

		//! Get StatusBar control.
		MGUI::CTRL_STATUSBAR& GetStatusBar (
			);
	
		//! Get VIEW_CANVAS managed by form.
		UI::VIEW_CANVAS* GetViewCanvas (
			) const;

		//! Get VIEW_CANVAS for "locator" managed by form.
		UI::VIEW_CANVAS* GetViewLocator (
			) const;

		//! Get attached viewable.
		GRE::VIEWABLE* GetViewable (
			) const;

		//! Enable or disable redraw of LegendView.
		void LegendEnableRedraw (
			bool enable
			);

		//! Determine if legend is visible.
		bool LegendIsVisible (
			bool CheckSidebar = true				//!< Check sidebar also
			) const;

		//! Redraw the legend if visible.
		void LegendRedraw (
			);

		//! Set whether legend is visible.
		void LegendSetVisible (
			bool visible
			);

		//! Get current Places height.
		//! @return Height in pixels.
		INT16 PlacesGetHeight (
			) const;

		//! Determine if Places is visible.
		bool PlacesIsVisible (
			bool CheckSidebar = true				//!< Check sidebar also
			) const;

		//! Set Places height.
		void PlacesSetHeight (
			INT16 height								//!< Height in pixels
			);

		//! Set whether Places is visible.
		//! Note: Legend must also be visible for Places to be.
		void PlacesSetVisible (
			bool visible
			);

		//! Get current locator height.
		//! @return Height in pixels.
		INT16 LocatorGetHeight (
			) const;

		//! Determine if locator is visible.
		bool LocatorIsVisible (
			bool CheckSidebar = true				//!< Check sidebar also
			) const;

		//! Set locator height.
		void LocatorSetHeight (
			INT16 height								//!< Height in pixels
			);

		//! Set whether locator is visible.
		//! Note: Legend must also be visible for locator to be.
		void LocatorSetVisible (
			bool visible
			);

		//! Get current magnifier height.
		//! @return Height in pixels.
		INT16 MagnifierGetHeight (
			) const;

		//! Get current magnifier zoom.
		INT16 MagnifierGetZoom (
			) const;

		//! Determine if magnifier is visible.
		bool MagnifierIsVisible (
			bool CheckSidebar = true				//!< Check sidebar also
			) const;

		//! Set magnifier height.
		void MagnifierSetHeight (
			INT16 height								//!< Height in pixels
			);

		//! Set magnifier zoom.
		void MagnifierSetZoom (
			INT16 zoom									//!< Zoom (2x, 4x, 8x, 16x)
			);

		//! Set whether magnifier is visible.
		//! Note: Legend must also be visible for magnifier to be.
		void MagnifierSetVisible (
			bool visible
			);

		//! Add items for "HotKeys".
		void MenuAddItemsHotKeys (
			MGUI::MENU& submenu						//!< SubMenu to add items to
			);
	
		//! Add items for "View".
		void MenuAddItemsView (
			MGUI::MENU& submenu						//!< SubMenu to add items to
			);

		//! Add submenu for "GPS".
		void MenuAddSubMenuGPS (
			MGUI::MENU& menu						//!< Menu to add to (usually a menu bar)
			);
	
		//! Add submenu for "HotKeys".
		void MenuAddSubMenuHotKeys (
			MGUI::MENU& menu						//!< Menu to add to (usually a menu bar)
			);
	
		//! Add submenu for "Macros".
		void MenuAddSubMenuMacros (
			MGUI::MENU& menu						//!< Menu to add to (usually a menu bar)
			);
	
		//! Add submenu for "Options".
		void MenuAddSubMenuOptions (
			MGUI::MENU& menu,						//!< Menu to add to (usually a menu bar)
			OPTIONSMENU flags = OPTIONSMENU_Default
			);
	
		//! Add submenu for "Tools" to specified menu.
		void MenuAddSubMenuTools (
			MGUI::MENU& menu						//!< Menu to add to (usually a menu bar)
			);
	
		//! Add submenu for "View".
		void MenuAddSubMenuView (
			MGUI::MENU& menu						//!< Menu to add to (usually a menu bar)
			);

		//! Add submenus to menubar.
		void MenuBarAddSubMenus (
			MGUI::MENU& MenuBar,						//!< Menu bar
			SUBMENUS submenus							//!< Which submenus to add
			);

		//! Create menubar with specified submenus.
		void MenuBarCreate (
			MGUI::SHELL& shell,						//!< Shell to attach menubar to
			SUBMENUS submenus = SUBMENUS_All,	//!< Which submenus to add
			bool AddSeparator = true				//!< Add separator line below menubar
			);

		//! Get MGUI::MENU for "Options", creating if necessary.
		//! May be used befor MenuBarCreate/MenuAddSubMenuOptions to add items to top of submenu,
		//! or afterward to add items to bottom of submenu.
		MGUI::MENU& MenuGetOptions ();
	
		//! Save "quick snapshot" of current view contents.
		ERRVALUE SaveSnapshotQuick (
			) const;

		//! Save "snapshot" of current view contents to raster.
		ERRVALUE SaveSnapshotToRaster (
			) const;

		//! Set delegate to call to add extra tools.
		void SetDelegateAddTools (
			DELEGATE_ADDTOOLS DelegateAddTools
			);

		//! Set whether to draw wireframe in 3D.
		void SetDrawWireframe3D (
			bool DrawWireframe3D
			);

		//! Set LISTPOS when use add layer
		void SetLayerListPos (
			GRE::LISTPOS ListPos
			);

		//! Set to previous view position.
		void SetPreviousView ();

		//! Set option to redraw view on any change.
		void SetRedrawOnAnyChange (
			bool RedrawOnAnyChange,
			bool redraw = true						//!< Redraw if changed and setting turned on
			);

		//! Set which "view" buttons will be created.
		//! If used, must be done before Create().
		void SetViewBtnsToCreate (
			VIEWBTNS viewbtns							//!< Buttons to be created 
			);

		//! Get current sidebar width.
		//! @return Width in pixels
		INT16 SidebarGetWidth (
			) const;

		//! Determine if sidebar is visible.
		bool SidebarIsVisible (
			) const;

		//! Set whether sidebar is visible.
		void SidebarSetVisible (
			bool visible
			);

		//! Set new sidebar width.
		void SidebarSetWidth (
			INT16 width									//!< Width in pixels
			);

		//! Clear coordinate display in scale/position report.
		void StatusClearCoord (
			COORDSOURCE CoordSource = COORDSOURCE_None
			);

		//! Display coordinates in status.
		void StatusDisplayCoord (
			COORDSOURCE CoordSource,						//!< Source for coordinates
			const DPOINT3D& point,							//!< Point for coordinates
			const SPATREF::COORDREFSYS& PointCRS,		//!< CRS of point
			const DPOINT3D *pPtConfidence,				//!< Point for determining confidence, typically one "pixel" away from specified point
			STATUSDISPLAY StatusDisplay,					//!< Display flags
			const UOM::UNIT *pUnitZ = 0					//!< Optional Z unit, if not set will check PointCRS or assume meters
			);

		COORDSOURCE StatusGetCoordSource () const;

		//! Determine if position indicator is visible.
		bool StatusIsPosVisible (
			int idx = -1
			) const;

		//! Set CRS for coordinate indicator on statusbar.
		void StatusSetCoordRefSys (
			unsigned idx,
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		//! Set coordinate "source" indicator for statusbar.
		void StatusSetCoordSource (
			COORDSOURCE CoordSource
			);

		//! Set format for geographic coordinate display.
		void StatusSetLatLonFormat (
			unsigned idx,
			DMSTYPE LatLonFormat
			);

		//! Set whether statusbar position indicator is visible.
		void StatusSetPosVisible (
			unsigned idx,
			bool visible
			);

		//! Set whether statusbar is visible.
		void StatusSetVisible (
			bool visible
			);

		//! Determine if stereo view indicators are visible when in stereo mode.
		bool StereoIsViewIndicatorVisible () const;

		//! Set whether stereo view indicators are visible.
		void StereoSetViewIndicatorVisible (
			bool visible								//!< Visible or not
			);

		//! Add 3D view control buttons to toolbar.
		void ToolBarAdd3dButtons ();

		//! Add "add layer/group" buttons to toolbar.
		void ToolBarAddAddButtons ();

		//! Add MacroScript buttons to toolbar.
		void ToolBarAddMacroScriptButtons ();

		//! Add "mark" buttons to toolbar.
		void ToolBarAddMarkButtons ();

		//! Add "tool" togglebuttons to toolbar for all defined tools.
		void ToolBarAddToolButtons ();

		//! Add "view" buttons (previous, zoom in, zoom out, zoom 1X, etc) to toolbar.
		void ToolBarAddViewButtons (
			VIEWBTNS buttons = VIEWBTN_Default		//! Which button to add
			);

		//! Update text on message line.
		void UpdateMessage (
			const MISTRING& string
			) const;

		//! Get current zoom pane height.
		//! @return Height in pixels.
		INT16 ZoomGetHeight (
			) const;

		//! Determine if zoom pane is visible.
		bool ZoomIsVisible (
			bool CheckSidebar = true				//!< Check sidebar also
			) const;

		//! Set whether zoom pane is visible.
		//! Note: Legend must also be visible for zoom pane to be.
		void ZoomSetVisible (
			bool visible
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT

		static FORM_VIEW* SML_CreateForm (
			int numargs,
			SML::ARG *args,
			SML::CONTEXT *context,
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		static SML::FLAG* SML_GetCreateFlags ();

	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		// Unimplemented
		FORM_VIEW (const FORM_VIEW&);
		FORM_VIEW& operator= (const FORM_VIEW&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_VIEW::CREATEFLAGS)
DEFINE_ENUM_OP_BITWISE(FORM_VIEW::VIEWBTNS)
DEFINE_ENUM_OP_BITWISE(FORM_VIEW::STATUSDISPLAY)
DEFINE_ENUM_OP_BITWISE(FORM_VIEW::SUBMENUS)
DEFINE_ENUM_OP_BITWISE(FORM_VIEW::OPTIONSMENU)
#endif

//===================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIFORMVIEW_H
