/**
 * \file mgui/formbtxt.h
 * \brief MGUI::FORM_BUTTONTEXT class definitions
 *
 * \if NODOC
 * $Id: formbtxt.h_v 1.16 2011/11/23 13:56:45 mju Exp $
 *
 * $Log: formbtxt.h_v $
 * Revision 1.16  2011/11/23 13:56:45  mju
 * Remove redundant hdr.
 *
 * Revision 1.15  2007/01/09 21:37:47  mju
 * Remove unused legacy create() method.
 *
 * Revision 1.13  2006/08/25 19:17:51  mju
 * Change to use ctrl_edit_string for mfc and add getTextCtrl.
 *
 * Revision 1.12  2006/07/21 20:49:39  mju
 * Add setToolTipString.
 *
 * Revision 1.1  2002/10/09 16:20:16  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMBTXT_H
#define	INC_MGUI_FORMBTXT_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif


namespace MGUI {

//===================================================================================================================
//! Form containing PUSHBUTTON and non-editable text field.
class FORM_BUTTONTEXT : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_BUTTONTEXT (
			);

		//! Clear the text field.
		void ClearText (
			) {
			m_Text.ClearValue(false);
			}

		//! Create with MISTRING/TEXTID label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			const MISTRING& label,					//!< Button label string
			int width = 0,								//!< Width in 'em' characters
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get reference to button control.
		CTRL_PUSHBUTTON& GetButton (
			) { return (m_Button); }

		//! Get reference to text control.
		//! @return non-editable control reference.
		CTRL_EDIT_STRING& GetTextCtrl (
			) { return (m_Text); }

		//! Set delegate to call when button pressed.
		void SetDelegateOnPressed (
			DELEGATE_VOID_NOPARMS delegate
			) { m_Button.SetDelegateOnPressed(delegate); }

		//! Set text to show in field.
		void SetText (
			const MISTRING& string
			);

		//! Set text to show as ToolTip.
		void SetToolTipString (
			const MISTRING& string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_EDIT_STRING m_Text;
		CTRL_PUSHBUTTON m_Button;

		void CreateText (int width, LAYOUT_SIZEALIGN sizealign);

		FORM_BUTTONTEXT (const FORM_BUTTONTEXT&);
		FORM_BUTTONTEXT& operator= (const FORM_BUTTONTEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMBTXT_H
