/*****************************************************************************
 * \file mi32/htmlmeasurement.h
 * \brief HTML::MEASUREMENT class.
 *
 * \if NODOC
 * $Log: htmlmeasurement.h_v $
 * Revision 1.3  2011/10/25 21:33:45  mju
 * Remove redundant hdrs.
 *
 * Revision 1.2  2007/09/07 14:50:18  dwilliss
 *
 * Revision 1.1  2007/08/29 15:42:16  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_HTMLMEASUREMENT_H
#define INC_MI32_HTMLMEASUREMENT_H

#ifndef INC_MI32_CSSPROPERTIES_H
#include <mi32/cssproperties.h>
#endif

#ifndef INC_MI32_SCRNUNIT_H
#include <mi32/scrnunit.h>
#endif

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGD {
	class CONTEXT;
	}
#endif

namespace HTML {
	class ELEMENT;

	//! A measurement of some kind in a STYLE
	class MEASUREMENT {
		public:

			enum RELATIVETO {
				RELATIVETO_NotAllowed = 0,
				RELATIVETO_ParentBlockWidth,
				RELATIVETO_ParentBlockHeight,
				RELATIVETO_TextHeight,
				RELATIVETO_ParentTextHeight,
				RELATIVETO_LineHeight,
				RELATIVETO_SelfHeight,
				RELATIVETO_SelfWidth
				};

			MEASUREMENT (
				CSS_PROPERTY property, //
				RELATIVETO relative = RELATIVETO_NotAllowed
				);

			MEASUREMENT(const MEASUREMENT&);

			bool operator==(const MEASUREMENT& rhs) const;
			bool operator!=(const MEASUREMENT& rhs) const;

			void Clear (
				) { 
				m_State = STATE_NotSet; 
				m_value = 0.0;
				}

			void Combine(const MEASUREMENT& rhs);

			const char* Parse(const char*);

			SCREENUNIT GetUnits (
				) const { return m_units; }

			double GetValue (
				) const { return m_value; }

			//! Get the value converted to pixels relative to the given device
			//! Note: This does not work if IsPercent() returns true.
			double GetValue (
				MGD::CONTEXT* gc,
				const ELEMENT* Element	// This element (used if size is relative)
				) const;

			//! Get the value of the measurement as a string that can be parsed
			//! as a valid CSS measurement.
			//! If the value is not set, it will return an empty string.
			MISTRING GetValueCSS (
				) const;

			bool IsAuto (
				) const { return (m_State == STATE_Auto); }

			bool IsPercent (
				) const { return (m_State == STATE_Percent); }

			bool IsSet (
				) const { return (m_State != STATE_NotSet); }

			void Set (
				double value,
				SCREENUNIT units
				) {
				m_value = value;
				m_units = units;
				m_State = STATE_Set;
				}

			void SetProperty (
				CSS_PROPERTY property
				) { m_Property = property; }

			void SetRelativeTo (
				RELATIVETO rel
				) { m_RelativeTo = rel; }

#ifndef GENERATING_DOXYGEN_OUTPUT
		private:
			enum STATE {
				STATE_NotSet = 0,
				STATE_Set,
				STATE_Percent,
				STATE_Auto,
				STATE_Inherit
				};
			STATE m_State;
			SCREENUNIT m_units;
			RELATIVETO m_RelativeTo;
			CSS_PROPERTY m_Property;
			double m_value;
#endif
		};

	} // end of HTML namespace

#endif
