/**
 * \file rvc/dbfieldconstraint.h
 * \brief RVC::DBFIELDCONSTRAINT enum definitions
 *
 * \if NODOC
 * $Id: dbfieldconstraint.h_v 1.2 2008/10/09 17:10:45 scowan Exp $
 *
 * $Log: dbfieldconstraint.h_v $
 * Revision 1.2  2008/10/09 17:10:45  scowan
 * Added bitwise ops to enum.
 *
 * Revision 1.1  2008/09/29 19:33:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_DBFIELDCONSTRAINT_H
#define	INC_RVC_DBFIELDCONSTRAINT_H

#ifndef INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>		// For DEFINE_ENUM_OP_BITWISE()
#endif

namespace RVC {

//! Database field constraint flags.
//! In file these are stored as a single UINT32 value.
enum DBFIELDCONSTRAINT {
	DBFIELDCONSTRAINT_Range_MinExclusive =		0x00000001,	//!< Value must be greater than minimum
	DBFIELDCONSTRAINT_Range_MaxExclusive =		0x00000002,	//!< Value must be less than maximum
	DBFIELDCONSTRAINT_Range_MinInclusive =		0x00000004,	//!< Value must be equal to or greater than minimum
	DBFIELDCONSTRAINT_Range_MaxInclusive =		0x00000008,	//!< Value must be equal to less than maximum

	DBFIELDCONSTRAINT_Default_Constant =		0x00000010,	//!< Default to constant value
	DBFIELDCONSTRAINT_Default_Previous =		0x00000020,	//!< Default to previous value if available, can combine with DefaultConstant or DefaultBlank
	DBFIELDCONSTRAINT_Default_Empty =			0x00000040,	//!< Default to not having any value, only meaningful if entry is required
	DBFIELDCONSTRAINT_Default_Increment =		0x00000080,	//!< Default to increment of previous value if available, not same as DBFIELDINFO::FLAG_AutoIncrement, which is applied when writing new record

	DBFIELDCONSTRAINT_Case_AllUpper =			0x00000100,	//!< String must be all uppercase
	DBFIELDCONSTRAINT_Case_AllLower =			0x00000200,	//!< String must be all lowercase
	DBFIELDCONSTRAINT_Case_FirstLetterCap =	0x00000400,	//!< Capitalize First Letter Of Every Word
	DBFIELDCONSTRAINT_Case_FirstWordCap =		0x00000800,	//!< Capitalize first letter of first word

	DBFIELDCONSTRAINT_PrimaryKey_MustExist =	0x00008000,	//!< Record related by value must exist in primary key table

	DBFIELDCONSTRAINT_PrimaryKey_AutoAdd =		0x00010000,	//!< If not in related primary key table, add it
	DBFIELDCONSTRAINT_PrimaryKey_PromptAdd =	0x00020000,	//!< If not in related primary key table, prompt to add
	DBFIELDCONSTRAINT_UI_ComboBox =				0x00040000,	//!< Show as a editable text with popup list
	DBFIELDCONSTRAINT_UI_OptionMenu =			0x00080000,	//!< Show as a popup list

	DBFIELDCONSTRAINT_NotNull =					0x00100000,	//!< Redundant with field flag, here for historical usage
	DBFIELDCONSTRAINT_UI_AutoComplete =			0x00400000,	//!< fill in based on PK values
	DBFIELDCONSTRAINT_ValidChars_Invert =		0x00800000,	//!< List of ValidChars is really invalid

	DBFIELDCONSTRAINT_ValueList_AllowDups =	0x01000000,	//!< Values stored comma-separated, chosen from list, duplicates allowed
	DBFIELDCONSTRAINT_ValueList_NoDups =		0x08000000,	//!< Values stored comma-separated, chosen from list, no duplicates

	// Mask values
	DBFIELDCONSTRAINT_Range_Min_MASK =			(DBFIELDCONSTRAINT_Range_MinExclusive | DBFIELDCONSTRAINT_Range_MinInclusive),
	DBFIELDCONSTRAINT_Range_Max_MASK =			(DBFIELDCONSTRAINT_Range_MaxExclusive | DBFIELDCONSTRAINT_Range_MaxInclusive),
	DBFIELDCONSTRAINT_Range_MASK =				(DBFIELDCONSTRAINT_Range_Min_MASK | DBFIELDCONSTRAINT_Range_Max_MASK),
	DBFIELDCONSTRAINT_Default_MASK =				(DBFIELDCONSTRAINT_Default_Constant | DBFIELDCONSTRAINT_Default_Previous | DBFIELDCONSTRAINT_Default_Empty | DBFIELDCONSTRAINT_Default_Increment),
	DBFIELDCONSTRAINT_Case_MASK =					(DBFIELDCONSTRAINT_Case_AllUpper | DBFIELDCONSTRAINT_Case_AllLower | DBFIELDCONSTRAINT_Case_FirstLetterCap | DBFIELDCONSTRAINT_Case_FirstWordCap),
	DBFIELDCONSTRAINT_PrimaryKey_MASK = 		(DBFIELDCONSTRAINT_PrimaryKey_MustExist | DBFIELDCONSTRAINT_PrimaryKey_AutoAdd | DBFIELDCONSTRAINT_PrimaryKey_PromptAdd),
	};

DEFINE_ENUM_OP_BITWISE(DBFIELDCONSTRAINT)

}	// End namespace RVC

#endif	// INC_RVC_DBFIELDCONSTRAINT_H
