/**
 * \file rvc/dbpoint.h
 * \brief interface for the RVC::DBASE_POINT class.
 *
 * \if NODOC
 * $Id: dbpoint.h_v 1.12 2010/05/12 21:07:45 mju Exp $
 *
 * $Log: dbpoint.h_v $
 * Revision 1.12  2010/05/12 21:07:45  mju
 * Simplify and specify parent objtype when construct base.
 *
 * Revision 1.11  2007/02/02 22:23:11  drb
 * Add Credentials to Open-As-Subobject
 *
 * Revision 1.10  2006/06/02 22:23:39  dwilliss
 * Changes to Join
 *
 * Revision 1.9  2005/09/23 17:38:37  dwilliss
 * Added CopyFrom method
 *
 * Revision 1.8  2003/10/07 17:34:47  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2002/04/08 16:00:54  scowan
 * Name change.
 *
 * Revision 1.3  2001/06/14 17:02:59  scowan
 * Add to RVC namespace.
 *
 * Revision 1.2  2001/05/25 17:01:24  scowan
 * Changed base class name.
 *
 * Revision 1.1  2001/05/24 15:52:35  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBPOINT_H
#define INC_RVC_DBPOINT_H

#ifndef INC_RVC_DBELEM_H
	#include <rvc/dbelem.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class VECTOR;
#endif

//! Manage RVC::OBJTYPE_Database_VectorPoint objects.
class DBASE_POINT : public RVC::DBASE_ELEMENT {
	public:

		//! Default constructor
		DBASE_POINT (
			): DBASE_ELEMENT(OBJTYPE_Vector,ELEMTYPE_Point) { }

		//! Copy a database as a point database under a vector.
		//! NOTE: This assumes that there is not already a point database
		//! under the vector.  Will get an error if there is.  This is used
		//! in database to vector conversion.
		ERRVALUE CopyFrom (
			const RVC::DBASE& source,				//!< Source database to copy
			const RVC::VECTOR& parent,				//!< Parent object to create database under
			MDLGPARENT dlgparent = 0				//!< Parent dialog handle, 0 for default
			);

		//! Join another database to this one and deal with possible nodes.
		ERRVALUE JoinPointDatabaseTo (
			RVC::DBASE_ELEMENT& DestDBase,
			const SIMPLE_ARRAY<INT32>& PointTransList,
			const SIMPLE_ARRAY<INT32>& NodeTransList,
			RVC::DBASE::COPYFLAG CopyFlags = RVC::DBASE::COPYFLAG_None,
			RVC::DBASE::JOINFLAGS JoinFlags = RVC::DBASE::JOINFLAG_None,
			INT32 *pNextElemID = 0,		//! Pointer to INT32 return next ElemID in
			ELEMTYPE SrcElemType = ELEMTYPE_Empty,  //! Will default to the default element type of the source database
			ELEMTYPE DestElemType = ELEMTYPE_Empty  //! Will default to the default element type of the destination database
			);

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_DBPOINT_H
