/**
 * \file rvc/spml.h 
 * \brief interface for the RVCSML class.
 *
 * \if NODOC
 * $Id: spml.h_v 1.7 2005/06/30 21:02:23 scowan Exp $
 *
 * $Log: spml.h_v $
 * Revision 1.7  2005/06/30 21:02:23  scowan
 * Added makeparms.
 *
 * Revision 1.6  2003/10/07 17:30:21  scowan
 * Object virtual method changes.
 *
 * Revision 1.5  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.3  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2001/06/15 17:26:48  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/29 16:09:08  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::SML rvc/spml.h
//!	\brief Manages RVC::OBJTYPE_Script_SML RVC objects.  
//!
//!	The interface currently wraps the MfSML functions.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Script_SML
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Script_SML.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder

#ifndef INC_RVC_SPML_H
#define INC_RVC_SPML_H

#ifndef INC_RVC_SCRIPT_H
	#include <rvc/script.h>
#endif

namespace RVC {

class SML : public RVC::SCRIPT {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif
		
		//! Default constructor
		SML (
			);

		//! Copy constructor
		SML (
			const RVC::SML& obj
			);

		//! Destructor
		virtual ~SML (
			);

		//! Assignment
		RVC::SML& operator= (
			const RVC::SML& rhs
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

class SML::MAKEPARMS : public SCRIPT::MAKEPARMS {
	public:
		// Default ctor
		MAKEPARMS (
			UINT32 FieldNum = 0,
			SCRIPT::USAGE Usage = SCRIPT::USAGE_Normal
			) : SCRIPT::MAKEPARMS(OBJTYPE_Script_SML, FieldNum, Usage) {}
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_SPML_H
