/**
 * \file rvc/themeprm.h 
 * \brief interface for the RVC::THEMEPARM class.
 *
 * \if NODOC
 * $Id: themeprm.h_v 1.15 2011/05/23 20:39:01 mju Exp $
 *
 * $Log: themeprm.h_v $
 * Revision 1.15  2011/05/23 20:39:01  mju
 * Add statsview.scaleLogH
 *
 * Revision 1.14  2007/09/06 17:19:04  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.13  2006/09/01 15:21:20  scowan
 * Moved data structure to here.
 *
 * Revision 1.12  2005/12/09 18:58:16  scowan
 * Updated to use credentials.
 *
 * Revision 1.11  2005/07/19 20:55:03  scowan
 * Fixed virtual method issue.
 *
 * Revision 1.10  2005/01/21 23:11:08  scowan
 * Fixed deprecated method.
 *
 * Revision 1.9  2003/10/07 17:30:17  scowan
 * Object virtual method changes.
 *
 * Revision 1.8  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.7  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.6  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2002/04/08 15:15:37  scowan
 * Removed pure indication.
 *
 * Revision 1.4  2002/04/05 22:34:30  scowan
 * Moved to derive from simple.
 *
 * Revision 1.3  2001/06/15 17:52:26  scowan
 * More const.
 *
 * Revision 1.2  2001/06/15 14:52:14  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/22 22:53:11  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::THEMEPARM rvc/themeprm.h
//!	\brief manages RVC::OBJTYPE_Theme objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Theme
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Theme
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Table

#ifndef INC_RVC_THEMEPRM_H
#define INC_RVC_THEMEPRM_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

namespace RVC {

class THEMEPARM : public RVC::SIMPLE {
	public:
	
		struct DATA {
			enum DISTRIB {
				DISTRIB_UserDefined = 0,
				DISTRIB_EqualCount,
				DISTRIB_EqualInterval,
				DISTRIB_Normalize,
				DISTRIB_GOVF
				};

			enum ATTRIBCOUNT {
				ATTRIBCOUNT_ByReference = 0,
				ATTRIBCOUNT_ElemSizes,
				ATTRIBCOUNT_AllRecords
				};

			enum ROUNDMODE {
				ROUNDMODE_None = 0,
				ROUNDMODE_Closest,
				ROUNDMODE_Down,
				ROUNDMODE_Up
				};

			enum COLORPATH {
				COLORPATH_Auto = 0,
				COLORPATH_RGB,
				COLORPATH_HISCW,
				COLORPATH_HISCCW,
				COLORPATH_HBSCW,
				COLORPATH_HBSCCW,
				COLORPATH_MaxAuto = COLORPATH_HBSCCW
				};

			struct STATSVIEW {
				UINT8 ScaleLog;						//!< Use logarithmic vertical scale for histogram
				UINT8 Envelope;						//!< Show histogram "envelope"
				UINT8 ScaleLogH;						//!< Use logarithmic horizontal scale for histogram
				};

			struct SPREAD {
				INT16 MinClass;						//!< Starting class for spread
				INT16 MaxClass;						//!< Ending class for spread
				UINT8 AllClasses;						//!< Set if applies to all classes
				UINT8 DoColor;							//!< Perform color spread
				UINT8 ColorPath;						//!< Path for color spread
				UINT8 PointByStyle;					//!< Point styles set individually
				UINT8 LineByStyle;					//!< Line styles set individually
				UINT8 PolyByStyle;					//!< Polygon styles set individually
				UINT8 DoPointSize;					//!< Spread point sizes
				UINT8 DoLineWidth;					//!< Spread line widths
				UINT8 PointSizeUnits;				//!< Units for point symbol size (SCREENUNIT_...)
				UINT8 LineWidthUnits;				//!< Units for line width (SCREENUNIT_...)
				UINT32 PointStyleFlags;				//!< Style flags (STYLEFLAG_...) for points
				UINT32 LineStyleFlags;				//!< Style flags (STYLEFLAG_...) for lines
				UINT32 PolyStyleFlags;				//!< Style flags (STYLEFLAG_...) for polygons
				COLOR StartColor;						//!< Starting spread color
				COLOR EndColor;						//!< Ending spread color
				double PointSizeStart;				//!< Starting point symbol size
				double PointSizeEnd;					//!< Ending point symbol size
				double PointSizeScale;				//!< Scale for point symbol size
				double LineWidthStart;				//!< Starting line width
				double LineWidthEnd;					//!< Ending line width
				double LineWidthScale;				//!< Scale for line width
				};

			INT16 NumClasses;							//!< Number of classes
			UINT8 DistribMode;						//!< Distribution mode, DISTRIB_...
			UINT8 CountMode;							//!< How attributes are counted, COUNTMODE_...
			UINT8 RoundMode;							//!< Rounding mode, ROUNDMODE_...
			UINT8 ExcludeFirstClass;				//!< Exclude first class from distribution
			UINT8 ExcludeLastClass;					//!< Exclude last class from distribution
			UINT8 ShowStats;							//!< Show statistics
			UINT8 ShowStyleParm;						//!< Show style options
			UINT8 LimitToDataRange;					//!< Limit class value ranges to existing data range
			double RoundTo;							//!< Interval to round to
			double lastmaxval;						//!< Last maximum range value
			STATSVIEW StatsView;
			SPREAD Spread;
				
			DATA () { memset(this, 0, sizeof(*this)); }
			};

		//! Default constructor
		THEMEPARM (
			);

		//! Copy constructor
		THEMEPARM (
			const RVC::THEMEPARM& rhs
			);

		//! Destructor
		virtual ~THEMEPARM (
			);

		//! Assignment
		RVC::THEMEPARM& operator= (
			const RVC::THEMEPARM& rhs
			);
			
		const DATA& GetThemeData (
			) const {return (m_ThemeData);}

		void SetThemeData (
			const DATA& ThemeData
			) {m_ThemeData = ThemeData;}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DATA m_ThemeData;

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_THEMEPRM_H
