/**
 * \file rvc/thumbnal.h 
 * \brief interface for the RVC::THUMBNAIL class.
 *
 * \if NODOC
 * $Id: thumbnal.h_v 1.12 2006/06/28 15:52:13 scowan Exp $
 *
 * $Log: thumbnal.h_v $
 * Revision 1.12  2006/06/28 15:52:13  scowan
 * Changed make to take a non-const objitem.
 *
 * Revision 1.11  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.10  2005/12/09 17:30:01  scowan
 * Updated to use credentials.
 *
 * Revision 1.9  2005/07/18 15:49:49  scowan
 * Updated to current standard.
 *
 * Revision 1.8  2003/10/07 17:30:16  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/15 17:51:19  scowan
 * More const.
 *
 * Revision 1.3  2001/06/15 14:58:05  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 21:17:33  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 16:25:10  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::THUMBNAIL rvc/thumbnal.h
//!	\brief manages RVC::OBJTYPE_Thumbnail objects.  
//!
//!	The interface currently wraps the MfThumbnail functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Thumbnail.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Thumbnail.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_TIN
//!		- RVC::OBJTYPE_Shape
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_HyperSpectral

#ifndef INC_RVC_THUMBNAL_H
#define INC_RVC_THUMBNAL_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

namespace RVC {

class THUMBNAIL : public RVC::OBJECT {
	public:

		//! Default constructor
		THUMBNAIL (
			);

		//! Copy constructor
		THUMBNAIL (
			const RVC::THUMBNAIL& obj
			);

		//! Destructor
		virtual ~THUMBNAIL (
			);

		//! Assignment
		RVC::THUMBNAIL& operator= (
			const RVC::THUMBNAIL& rhs
			);

		//! Make a thumbnail rvc object from a given parent with a DESCRIPTOR
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a thumbnail rvc object from a preset OBJITEM
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const char* Source = 0,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_THUMBNAL_H
