/**
 * \file rvc/uiformrastercompression.h
 * \brief RVC::UI::FORM_RASTER_COMPRESSION class definitions
 *
 * \if NODOC
 * $Id: uiformrastercompression.h_v 1.6 2007/09/20 15:08:06 mju Exp $
 *
 * $Log: uiformrastercompression.h_v $
 * Revision 1.6  2007/09/20 15:08:06  mju
 * Add members.
 *
 * Revision 1.5  2007/09/19 13:55:07  mju
 * Add iniRead/Write.
 *
 * Revision 1.4  2007/04/27 23:17:27  dfriberg
 * Add Get method
 *
 * Revision 1.3  2007/01/08 14:03:28  mju
 * Fix comment.
 *
 * Revision 1.2  2006/06/26 20:30:45  mju
 * Fix header comment for filename.
 *
 * Revision 1.1  2006/06/26 16:16:48  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMRASTERCOMPRESSION_H
#define	INC_RVC_UIFORMRASTERCOMPRESSION_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

namespace RVC {
namespace UI {

//! Form for setting raster internal compression type and quality.
class FORM_RASTER_COMPRESSION : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style enumeration.	
		enum FORMSTYLE {
			FORMSTYLE_Default =		0x00,
			FORMSTYLE_NoLossy =		0x01,		//!< Don't allow lossy compression choices
			FORMSTYLE_NoReadOnly =	0x02		//!< Don't allow compression which does not support overwrite
			};
	
		//! Constructor.
		//! Defaults to a celltype of RVC::IMAGE::CELLTYPE_Unsigned_8 for menu generation purposes
		FORM_RASTER_COMPRESSION (
			);

		//! Destructor.
		virtual ~FORM_RASTER_COMPRESSION (
			);
			
		//! Create the form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			FORMSTYLE Style = FORMSTYLE_Default,			//!< Form style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get currently selected compression quality/ratio.
		//! @return Compression quality/ratio.
		UINT16 GetCompressionQualityRatio (
			);

		//! Get currently selected compression type.
		RVC::RASTER::COMPTYPE GetCompressionType (
			) const;

		//! Retrieve combo box control for form.
		MGUI::CTRL_COMBOBOX& GetComboBox (
			) { return (m_CbxType.GetComboBox()); }

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_CbxType.GetLabel()); }

		//! Read settings from INI file.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify									//!< Call delegates if changed
			);

		//! Write settings to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const;

		//! Determine if lossy compression type is selected.
		bool IsLossy (
			) const;

		//! Set cell type for limiting compression choices.
		//! If current compression type not allowed for specified CELLTYPE
		//! the selection will be reset to 'uncompressed'.
		//! Create() must be called before using this method.
		void SetCellType (
			RVC::IMAGE::CELLTYPE CellType
			);

		//! Set selected compression parameters.
		//! If specified values are not allowed for current CELLTYPE the
		//! selection will be left unchanged.
		//! Create() must be called before using this method.
		void SetCompression (
			RVC::RASTER::COMPTYPE CompType,					//!< Compression type
			UINT16 CompQualityRatio = 0						//!< Compression quality or ratio, 0 for maximum quality/lossless
			);

		//! Set delegate to call when compression type changed.
		void SetDelegateOnChangeCompressionType (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeCompressionType = delegate; }

		//! Set delegate to call when quality/ratio value changed.
		void SetDelegateOnChangeQualityRatio (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set cell type and compression selection from RASTER::MAKEPARMS.
		void SetFromMakeParms (
			const RVC::RASTER::MAKEPARMS& MakeParms
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnChangeCompressionType;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeQualityRatio;
		FORMSTYLE m_FormStyle;
		RVC::IMAGE::CELLTYPE m_CellType;
		MGUI::FORM_COMBOBOX m_CbxType;
		MGUI::FORM_EDIT_NUMBER m_EditQuality;
		MGUI::FORM_EDIT_NUMBER m_EditRatio;

		void OnCbxType ();
		void OnEditRatio ();
		void UpdateUI ();
		#endif // GENERATING_DOXYGEN_OUTPUT

	};
	
DEFINE_ENUM_OP_BITWISE(FORM_RASTER_COMPRESSION::FORMSTYLE)

}		// End namespace UI
}		// End namespace RVC


#endif		// INC_RVC_UIFORMRASTERCOMPRESSION_H
