/**
 *  \file rvc/uilabelframestyle.h
 *  \brief RVC User Interface for editing label frame style
 *
 *  \if NODOC
 *  $Id: uilabelframestyle.h_v 1.1 2006/09/20 15:10:50 mju Exp $
 *
 *  $Log: uilabelframestyle.h_v $
 *  Revision 1.1  2006/09/20 15:10:50  mju
 *  Initial revision
 *
 *  \endif
 **/

#ifndef INC_RVC_UILABELFRAMESTYLE_H
#define INC_RVC_UILABELFRAMESTYLE_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_RVC_UISTYLE_H
#include <rvc/uistyle.h>
#endif

#ifndef INC_MI32_LABLSTYL_H
#include <mi32/lablstyl.h>
#endif


namespace RVC {
namespace UI {

class DLG_LABELFRAMESTYLE : public MGUI::DLGSHELL {
	public:
		DLG_LABELFRAMESTYLE (
			LABELFRAMEPARMS& parms,
			const RVC::OBJITEM& StyleObjItem,
			double CurrentScale
			);

		~DLG_LABELFRAMESTYLE();

		const RVC::OBJITEM& GetStyleObjItem (
			) const {
			return m_StyleObjItem;
			}
		
		LABELFRAMEPARMS& GetFrameParms (
			) {
			return (m_parms);
			}	

		// Used by pages to make sure they have current style
		void Update();

	private:
	#ifndef GENERATING_DOXYGEN_OUTPUT

		class PAGE_FRAME : public MGUI::LAYOUT_PAGE {
			public:
				PAGE_FRAME (
					DLG_LABELFRAMESTYLE* dlg,
					double CurrentMapScale
					);

				~PAGE_FRAME() {}
				void Update();

				virtual void OnInitPage();
				virtual void v_OnSetActive();
			private:
				MGUI::CTRL_LABEL m_CtrlLabelPct1;
				MGUI::CTRL_LABEL m_CtrlLabelPct2;
				MGUI::CTRL_LABEL m_CtrlLabelPct3;
				MGUI::CTRL_LABEL m_CtrlLabelPct4;
				MGUI::FORM_COMBOBOX m_CbxFrameShape;
				MGUI::CTRL_TOGGLEBUTTON m_CtrlMarginToggle;
				MGUI::FORM_EDIT_NUMBER m_CtrlTMargin;
				MGUI::FORM_EDIT_NUMBER m_CtrlBMargin;
				MGUI::FORM_EDIT_NUMBER m_CtrlLMargin;
				MGUI::FORM_EDIT_NUMBER m_CtrlRMargin;
				RVC::UI::FORM_EDIT_STYLE m_StyleForm;
				DLG_LABELFRAMESTYLE* m_dlg;
				double m_CurrentMapScale;

				void SetAllMargins (double val);
				void OnChangeTMargin ();
				void OnChangeBMargin ();
				void OnChangeLMargin ();
				void OnChangeRMargin ();
				void OnToggleMarginLock ();
				void OnShapeChange ();
				void UpdateUI ();
			};

		class PAGE_LEADER : public MGUI::LAYOUT_PAGE {
			public:
				PAGE_LEADER (
					DLG_LABELFRAMESTYLE* dlg,
					double CurrentMapScale
					);

				~PAGE_LEADER() {}
				virtual void OnInitPage();
				virtual void v_OnSetActive();
				void Update();
			private:
				MGUI::CTRL_TOGGLEBUTTON m_CtrlBalloonLeader;
				RVC::UI::FORM_EDIT_STYLE m_StyleForm;
				DLG_LABELFRAMESTYLE* m_dlg;
				double m_CurrentMapScale;

				void OnTglBalloonLeaders();
				void UpdateUI();
			};

		class PAGE_SAMPLE : public MGUI::LAYOUT_PAGE {
			public:
				PAGE_SAMPLE (
					DLG_LABELFRAMESTYLE* dlg,
					double CurrentMapScale
					);

				~PAGE_SAMPLE() {}

				virtual void OnInitPage();
				virtual void v_OnSetActive();

			private:
				DLG_LABELFRAMESTYLE* m_dlg;
				double m_CurrentMapScale;

			};

		const RVC::OBJITEM& m_StyleObjItem;
		LABELFRAMEPARMS m_parms;
		LABELFRAMEPARMS& m_original;
		MGUI::LAYOUT_BOOK m_Book;
		PAGE_FRAME m_PageFrame;
		PAGE_LEADER m_PageLeader;
		PAGE_SAMPLE m_PageSample;

		virtual ERRVALUE v_CreateContent();
		virtual void v_OnApply();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace UI
}	// End namespace RVC


#endif // INC_RVC_UILABELFRAMESTYLE_H
